unit uMHA_ResourceFile_OPS;

(*
Resource files are used to speed operation. When an instrument is called for
display for data-entry, the resource file is quicker to provide Questions,
Answers, Introductions, Sections and Rules than a broker call.

Scale Groups, Scales and Keys are not needed in the resource file: This are
for results only and broker are used for that information.

The format of these files is NOT the same as import/export files.

*)

interface

uses
  Classes,
  Sysutils,
  fIA_dialogs,
  uIA_AvailableInstrument,
  uIA_Instrument;

type
  TResourceFile_OPS = class
  private
    function ResourceFileExists(aAvail: TAvailableInstrument): boolean;
    procedure DeleteOutdatedFiles(aAvail: TAvailableInstrument);
    procedure DeleteUnlicensedFiles(aAvail: TAvailableInstrument);
    procedure UpdateInstrumentNameFile(aName, aPrintTitle: string);
  protected
  public
    procedure CleanAndUpdateResourceFiles;
    function ReadResourceFile(var aInstrument: TInstrument): Boolean;
    procedure WriteResourceFile(aInstrument: TInstrument);
  published
  end; { TResourceFile_OPS }

var
  ResourceFile_OPS            : TResourceFile_OPS;

implementation

uses
  uMHA_Registry,
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_LocalFiles,
  uMHA_Instrument_OPS,
  uMHA_ResourceFile_IO,
  fMHA_Progress,
  uMHA_DM;

procedure TResourceFile_OPS.CleanAndUpdateResourceFiles;
var
  i                           : integer;
  DisplayMsg                  : Boolean;
  aInstrument                 : TInstrument;
begin
  if not (DataBaseMode = dbVista) then
    Exit;

  DisplayMsg := False;

  frmMHA_Progress := TfrmMHA_Progress.Create(nil);
  try
    frmMHA_Progress.SetupProgressBar('Loading and updating instruments. . .',
      Instrument_OPS.ListOfInstruments.Count);

    for i := 0 to Instrument_OPS.ListOfInstruments.Count - 1 do
      begin
        frmMHA_Progress.IncreaseProgressBar;

        DeleteOutdatedFiles(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]));

        if not TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).CopyrightStatusIsOK then
          DeleteUnlicensedFiles(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]));

        if TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).CanBeAdministered then
          begin

            if not ResourceFileExists(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i])) then
              begin
                aInstrument := TInstrument.Create;
                aInstrument.InstrumentName := TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName;
                Instrument_OPS.Load(aInstrument);

                // This will build the resource file and keep track of the latest version of
                // YS_MHA_AUX.dll.
                ResourceFile_IO.BuildInstrumentResourceFile(
                  LocalFiles.InstrumentResourceFileName(aInstrument.InstrumentName,
                  aInstrument.LastEditFMDate),
                  aInstrument);

                UpdateInstrumentNameFile(aInstrument.InstrumentName,
                  aInstrument.PrintTitle);

                aInstrument.Free;
                DisplayMsg := True;

              end;
          end;
      end; //for i...

  finally
    frmMHA_Progress.Free;
  end; //try

  

  if gvUseScreenReader and DisplayMsg then
    MsgDlgFrm('Files now updated.', 'Files now updated.',
      mtInformation, [mbOk], 0);

end;

procedure TResourceFile_OPS.UpdateInstrumentNameFile(aName, aPrintTitle: string);
// Used in Instrument Select form with hints
var
  TempStringList              : TStringList;
begin
  TempStringList := TStringList.Create;
  if FileExists(LocalFiles.InstrumentFilesPath + '\Names.txt') then
    begin
      TempStringList.LoadFromFile(LocalFiles.InstrumentFilesPath + '\Names.txt');
      TempStringList.Add(aName + EQ + aPrintTitle);
      TempStringList.Sort;
      TempStringList.SaveToFile(LocalFiles.InstrumentFilesPath + '\Names.txt');
    end
  else
    begin
      TempStringList.Add(aName + EQ + aPrintTitle);
      TempStringList.SaveToFile(LocalFiles.InstrumentFilesPath + '\Names.txt');
    end;

  TempStringList.Free;
end;


procedure TResourceFile_OPS.WriteResourceFile(aInstrument: TInstrument);
begin
  ResourceFile_IO.BuildInstrumentResourceFile(
    LocalFiles.InstrumentResourceFileName(aInstrument.InstrumentName,
    aInstrument.LastEditFMDate),
    aInstrument);
end;

procedure TResourceFile_OPS.DeleteOutdatedFiles(aAvail: TAvailableInstrument);

  function FileIsOutdated(AFileName: string; ALastEditFMDate: string): BOOLEAN;
  var
    tmpFileDate               : string;
  begin
    tmpFileDate := Piece(Piece(AFileName, '.', 1), FILE_NAME_DELIMITER, 2);

    ALastEditFMDate := Piece(ALastEditFMDate, '.', 1)
      + Piece(ALastEditFMDate, '.', 2);

    Result := not (tmpFileDate = ALastEditFMDate);

  end; //FileLastEditDate

var
  AFileNameStem               : string;
  sr                          : TSearchRec;
begin
  AFileNameStem := LocalFiles.InstrumentFilesPath + '\' +
    aAvail.AvailInstrumentName;

  // if there are any off-line files for the test, then delete them if the
  // last edit date doesn't match

  if (FindFirst(AFileNameStem + FILE_NAME_DELIMITER + '*.' +
    RESOURCE_FILE_EXT, faAnyFile, sr) = 0) then
    begin
      if FileIsOutdated(sr.Name, aAvail.LastEditDate) then
        DeleteFile(LocalFiles.InstrumentFilesPath + '\' + sr.Name);
      while (FindNext(sr) = 0) do
        if FileIsOutdated(sr.Name, aAvail.LastEditDate) then
          DeleteFile(LocalFiles.InstrumentFilesPath + '\' + sr.Name);
    end;

  FindClose(sr);

end; //DeleteOutdatedFiles


function TResourceFile_OPS.ResourceFileExists(aAvail: TAvailableInstrument): boolean;
var
  sr                          : TSearchRec;
begin
  Result := False;

  if (FindFirst(LocalFiles.InstrumentResourceFileName(
    aAvail.AvailInstrumentName, aAvail.LastEditDate),
    faAnyFile, sr) = 0) then
    Result := True;

  FindClose(sr);
end;

procedure TResourceFile_OPS.DeleteUnlicensedFiles(aAvail: TAvailableInstrument);
var
  sr                          : TSearchRec;
begin
  if (FindFirst(LocalFiles.InstrumentFilesPath + '\'
    + aAvail.AvailInstrumentName + FILE_NAME_DELIMITER + '.' + RESOURCE_FILE_EXT, faAnyFile, sr) = 0) then
    begin
      DeleteFile(LocalFiles.InstrumentFilesPath + '\' + sr.Name);
      while (FindNext(sr) = 0) do
        DeleteFile(LocalFiles.InstrumentFilesPath + '\' + sr.Name);
    end;

  FindClose(sr);
end;

function TResourceFile_OPS.ReadResourceFile(var aInstrument: TInstrument): BOOLEAN;
var
  tmpFileName, tmpName        : string;
  sr                          : TSearchRec;
begin

  tmpName := aInstrument.InstrumentName;

  Result := False;

  if (FindFirst(LocalFiles.InstrumentFilesPath + '\' +
    tmpName + FILE_NAME_DELIMITER + '*.' + RESOURCE_FILE_EXT,
    faAnyFile, sr) = 0) then
    begin
      tmpFileName := LocalFiles.InstrumentFilesPath + '\' + sr.Name;
      FindClose(sr);

      try
        begin
          ResourceFile_IO.ReadInstrumentResourceFile(
            TmpFileName, aInstrument);
          Result := True;
        end;
      except
        //If you like, you can tell the user "I cant read options"
      end
    end; //if (FindFirst...
end;

end.

