unit uMHA_AUX_SPSI;

interface

uses
  SysUtils,
  DateUtils;

function ScoreAnswers_SPSI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_SPSI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The Social Problem-Solving Inventory, Revised, Short Form (SPSI-R S) has
  25 questions, all radio groups, with five scales, and a total score.

  The raw scores are converted to "Standard Scores" with a range of 25 to 165.
  (Not standard to me.)
*}

function ScoreAnswers_SPSI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  PPO_YA_Max                  : integer = 131;
  PPO_YA_Min                  : integer = 54;
  PPO_MA_Max                  : integer = 125;
  PPO_MA_Min                  : integer = 47;
  PPO_EA_Max                  : integer = 135;
  PPO_EA_Min                  : integer = 52;

  NPO_YA_Max                  : integer = 138;
  NPO_YA_Min                  : integer = 74;
  NPO_MA_Max                  : integer = 162;
  NPO_MA_Min                  : integer = 78;
  NPO_EA_Max                  : integer = 146;
  NPO_EA_Min                  : integer = 79;

  RPS_YA_Max                  : integer = 136;
  RPS_YA_Min                  : integer = 56;
  RPS_MA_Max                  : integer = 127;
  RPS_MA_Min                  : integer = 58;
  RPS_EA_Max                  : integer = 137;
  RPS_EA_Min                  : integer = 58;

  ICS_YA_Max                  : integer = 149;
  ICS_YA_Min                  : integer = 73;
  ICS_MA_Max                  : integer = 162;
  ICS_MA_Min                  : integer = 81;
  ICS_EA_Max                  : integer = 157;
  ICS_EA_Min                  : integer = 77;

  AS_YA_Max                   : integer = 141;
  AS_YA_Min                   : integer = 78;
  AS_MA_Max                   : integer = 155;
  AS_MA_Min                   : integer = 84;
  AS_EA_Max                   : integer = 157;
  AS_EA_Min                   : integer = 76;

  Total_YA_Max                : integer = 140;
  Total_YA_Min                : integer = 39;
  Total_MA_Max                : integer = 127;
  Total_MA_Min                : integer = 32;
  Total_EA_Max                : integer = 139;
  Total_EA_Min                : integer = 29;

var

  PPO, NPO, RPS, ICS, AS1, Total:
    Double;

  PPOCount, NPOCount, RPSCount, ICSCount, AS1Count,
    Age,
    Missing,
    Count, i, x,
    TempAnswer                : Integer;

  DOB,
    TempString,
    TempResult
                              : string;

  function GetStandardScore(Max, Min: integer; Raw: Double; Reverse: boolean): string;
  var
    Score                     : double;
  begin
    if Raw < 0 then
      begin
        Result := '0'; //Too many missing answers. Don't use a char as this will
        // not display the graph properly.
      end
    else
      begin
        Score := (raw / 20) * (Max - Min) + Min;

        if Reverse then
          Total := Total + (20 - Raw) / 5
        else
          Total := Total + (Raw / 5);

        Result := FloatToStrF(Score, ffFixed, 10, 0);
      end;
  end;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'SPSI' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Missing := 0; // Max allowed is two
  PPOCount := 0;
  NPOCount := 0;
  RPSCount := 0;
  ICSCount := 0;
  AS1Count := 0;
  PPO := 0;
  NPO := 0;
  RPS := 0;
  ICS := 0;
  AS1 := 0;
  Total := 0;
  Age := 0;

  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempString := Piece(Piece(TempResult, U, 1), EQ, 2);

       // Count the number of skipped (Blank) answers
      if (TempString = '') or (TempString = ' ') or
        (TempString = '-1') or
        (TempString = NOT_ASKED) or
        (TempString = SKIPPED) or
        (TempString = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempString = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          Inc(Missing);
          if Missing > 2 then
            begin
              Result := 'Too many questions were not answered. The maximum allowed are two.';
              exit;
            end;
        end
      else
        begin
          TempAnswer := StrToIntDef(TempString, -1);
          if (i = 4) or (i = 5) or (i = 9) or (i = 13) or (i = 15) then
            begin
              PPO := PPO + TempAnswer;
              Inc(PPOCount);
            end;

          if (i = 1) or (i = 3) or (i = 7) or (i = 8) or (i = 11) then
            begin
              NPO := NPO + TempAnswer;
              Inc(NPOCount);
            end;

          if (i = 12) or (i = 16) or (i = 19) or (i = 21) or (i = 23) then
            begin
              RPS := RPS + TempAnswer;
              Inc(RPSCount);
            end;

          if (i = 2) or (i = 14) or (i = 20) or (i = 24) or (i = 25) then
            begin
              ICS := ICS + TempAnswer;
              Inc(ICSCount);
            end;

          if (i = 6) or (i = 10) or (i = 17) or (i = 18) or (i = 22) then
            begin
              AS1 := AS1 + TempAnswer;
              Inc(AS1Count);
            end;
        end;
    end;

  // Take care of a missing score, only one item can be missed in a scale.
  // Add the average of the answered items in that scale.
  if PPOCount = 4 then
    PPO := PPO + PPO / 4;
  if NPOCount = 4 then
    NPO := NPO + NPO / 4;
  if RPSCount = 4 then
    RPS := RPS + RPS / 4;
  if ICSCount = 4 then
    ICS := ICS + ICS / 4;
  if AS1Count = 4 then
    AS1 := AS1 + AS1 / 4;

  // Too many missing items in a scale; create a minus score
  if PPOCount = 3 then
    begin
      Total := -200; // Any large number will do
      PPO := -1;
    end;
  if NPOCount = 3 then
    begin
      Total := -200;
      NPO := -1;
    end;
  if RPSCount = 3 then
    begin
      Total := -200;
      RPS := -1;
    end;
  if ICSCount = 3 then
    begin
      Total := -200;
      ICS := -1;
    end;
  if AS1Count = 3 then
    begin
      Total := -200;
      AS1 := -1;
    end;

  // Determine age: DOB Format: JAN 17, 1950
  DOB := '';
  DOB := DateStrToMMDDYYYYFormat(PatientDOB, '/');

  if DOB = 'Error' then
    begin
      Result := 'The standard scores require the veteran''s age in its calculation ' +
        'which could not be determined.';
      exit;
    end;

  Age := YearsBetween(Today, StrToDateDef(DOB, Today));

  // Default statement
  Result := 'The standard scores require the veteran''s age in its calculation ' +
    'which could not be determined.';

  if (Age > 16) and (Age < 40) then
    begin
  // Note: "PPO" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
      Result :=
        'PPO^^' + GetStandardScore(PPO_YA_Max, PPO_YA_Min, PPO, False) + RECORD_END +
        'NPO^^' + GetStandardScore(NPO_YA_Max, NPO_YA_Min, NPO, True) + RECORD_END +
        'RPS^^' + GetStandardScore(RPS_YA_Max, RPS_YA_Min, RPS, False) + RECORD_END +
        'ICS^^' + GetStandardScore(ICS_YA_Max, ICS_YA_Min, ICS, True) + RECORD_END +
        'AS^^' + GetStandardScore(AS_YA_Max, AS_YA_Min, AS1, True) + RECORD_END +
        'Total^^' + GetStandardScore(Total_YA_Max, Total_YA_Min, Total, False) + RECORD_END;
    end;

  if (Age > 39) and (Age < 60) then
    begin
      Result :=
        'PPO^^' + GetStandardScore(PPO_MA_Max, PPO_MA_Min, PPO, False) + RECORD_END +
        'NPO^^' + GetStandardScore(NPO_MA_Max, NPO_MA_Min, NPO, True) + RECORD_END +
        'RPS^^' + GetStandardScore(RPS_MA_Max, RPS_MA_Min, RPS, False) + RECORD_END +
        'ICS^^' + GetStandardScore(ICS_MA_Max, ICS_MA_Min, ICS, True) + RECORD_END +
        'AS^^' + GetStandardScore(AS_MA_Max, AS_MA_Min, AS1, True) + RECORD_END +
        'Total^^' + GetStandardScore(Total_MA_Max, Total_MA_Min, Total, False) + RECORD_END;
    end;

  if (Age > 59) then
    begin
      Result :=
        'PPO^^' + GetStandardScore(PPO_EA_Max, PPO_EA_Min, PPO, False) + RECORD_END +
        'NPO^^' + GetStandardScore(NPO_EA_Max, NPO_EA_Min, NPO, True) + RECORD_END +
        'RPS^^' + GetStandardScore(RPS_EA_Max, RPS_EA_Min, RPS, False) + RECORD_END +
        'ICS^^' + GetStandardScore(ICS_EA_Max, ICS_EA_Min, ICS, True) + RECORD_END +
        'AS^^' + GetStandardScore(AS_EA_Max, AS_EA_Min, AS1, True) + RECORD_END +
        'Total^^' + GetStandardScore(Total_EA_Max, Total_EA_Min, Total, False) + RECORD_END;
    end;
end;


function ProgressNote_SPSI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  if NumberofPieces(ScoreString, RECORD_END) = 1 then
    begin
      Result := ScoreString;
      exit;
    end;

  Result := 'SPSI-R Standard Scores' + #13 + #13 + Indent +

  'PPO - Positive Problem Orientation: ' + Piece(Piece(ScoreString, RECORD_END, 1), U, 3) + #13 + Indent +
    'NPO - Negative Problem Orientation: ' + Piece(Piece(ScoreString, RECORD_END, 2), U, 3) + #13 + Indent +
    'RPS - Rational Problem Solving:     ' + Piece(Piece(ScoreString, RECORD_END, 3), U, 3) + #13 + Indent +
    'ICS - Impulsive/Careless Style:     ' + Piece(Piece(ScoreString, RECORD_END, 4), U, 3) + #13 + Indent +
    'AS  - Avoidance Style:              ' + Piece(Piece(ScoreString, RECORD_END, 5), U, 3) + #13 + #13 + Indent +
    '      Total:                        ' + Piece(Piece(ScoreString, RECORD_END, 6), U, 3) + #13 + #13 + Indent +

  'A standard score has a mean of 100, thus a score above 100 indicates above average use of ' +
    'the problem-solving style. A zero score indicates the scale had too many skipped items.' + #13 + #13 + Indent +

  'Questions and Answers:';

end;

end.

